#nvpl_scalapack-config.cmake
#
# Exported VARIABLES
#  * NVPL_BLACS_MPI_INTERFACES - List of available MPI Interface libs
#  * NVPL_SCALAPACK_INT_TYPES - List of available integer ABIs
#  * nvpl_scalapack_VERSION - Version of installed package
#  * nvpl_scalapack_INCLUDE_DIR - Location of nvpl_scalapack headers
#  * nvpl_scalapack_LIBRARY_DIR - Location of nvpl_scalapack libraries
#
# Interface targets
#  * nvpl::blacs_<int>_<mpi>
#      - BLACS interface where
#          <int> is from NVPL_SCALAPACK_INT_TYPES
#          <mpi> is from NVPL_BLACS_MPI_INTERFACES
#
#  * nvpl::scalapack_<int>
#       - ScaLAPACK library where
#           <int> is from NVPL_SCALAPACK_INT_TYPES


set(nvpl_scalapack_VERSION "0.2.1")
# build: 460


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was PackageConfig.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

# Dependencies
include(CMakeFindDependencyMacro)
if(NOT nvpl_blas_FOUND)
    find_dependency(nvpl_blas)
endif()
if(NOT nvpl_lapack_FOUND)
    find_dependency(nvpl_lapack)
endif()

# Find header and lib directories via known header and lib names
set(_nvpl_scalapack_search_header_name nvpl_scalapack.h)
set(_nvpl_scalapack_search_lib_name libnvpl_scalapack_lp64.so)
get_filename_component(_nvpl_scalapack_search_prefix "${CMAKE_CURRENT_LIST_DIR}/../../" ABSOLUTE)

find_path(nvpl_scalapack_INCLUDE_DIR
          NAMES ${_nvpl_scalapack_search_header_name}
          PATHS ${_nvpl_scalapack_search_prefix}
          PATH_SUFFIXES
            ../include
            ../../include # Debian/Ubuntu aarch64
          NO_DEFAULT_PATH
          NO_CACHE) # Set cache variable after path normalization

find_path(nvpl_scalapack_LIBRARY_DIR
          NAMES ${_nvpl_scalapack_search_lib_name}
          PATHS ${_nvpl_scalapack_search_prefix}
          PATH_SUFFIXES lib lib64
          DOC "Location of nvpl_scalapack libraries"
          NO_DEFAULT_PATH)

# Check headers and library directories are found
if(NOT EXISTS "${nvpl_scalapack_INCLUDE_DIR}")
    message(FATAL_ERROR "Header directory containing file ${_nvpl_scalapack_search_header_name} was not found relative to ${CMAKE_CURRENT_LIST_DIR}!")
endif()
if(NOT EXISTS "${nvpl_scalapack_LIBRARY_DIR}")
    message(FATAL_ERROR "Library directory containing file ${_nvpl_scalapack_search_lib_name} was not found relative to ${CMAKE_CURRENT_LIST_DIR}!")
endif()

# Normalize nvpl_scalapack_INCLUDE_DIR and set cache variable
get_filename_component(nvpl_scalapack_INCLUDE_DIR "${nvpl_scalapack_INCLUDE_DIR}" ABSOLUTE)
set(nvpl_scalapack_INCLUDE_DIR ${nvpl_scalapack_INCLUDE_DIR} CACHE STRING "Location of nvpl_scalapack headers")

unset(_nvpl_scalapack_search_header_name)
unset(_nvpl_scalapack_search_lib_name)
unset(_nvpl_scalapack_search_prefix)

# Targets
if(NOT TARGET nvpl_scalapack_ilp64 OR NOT TARGET nvpl_scalapack_lp64)
    include("${CMAKE_CURRENT_LIST_DIR}/nvpl_scalapack-targets.cmake")
    include("${CMAKE_CURRENT_LIST_DIR}/nvpl_scalapack-static-targets.cmake" OPTIONAL)
endif()

set(NVPL_SCALAPACK_INT_TYPES lp64;ilp64)
set(NVPL_BLACS_MPI_INTERFACES mpich;openmpi3;openmpi4;openmpi5)

foreach(_int IN LISTS NVPL_SCALAPACK_INT_TYPES)
    foreach(_mpi IN LISTS NVPL_BLACS_MPI_INTERFACES)
        if(NOT TARGET nvpl::blacs_${_int}_${_mpi})
            set_target_properties(nvpl_blacs_${_int}_${_mpi} PROPERTIES IMPORTED_GLOBAL 1)
            add_library(nvpl::blacs_${_int}_${_mpi} ALIAS nvpl_blacs_${_int}_${_mpi})
        endif()
    endforeach()
    if(NOT TARGET nvpl::scalapack_${_int})
            set_target_properties(nvpl_scalapack_${_int} PROPERTIES IMPORTED_GLOBAL 1)
        add_library(nvpl::scalapack_${_int} ALIAS nvpl_scalapack_${_int})
    endif()
endforeach()
unset(_int)
unset(_mpi)

check_required_components(nvpl_scalapack)

if(NOT ${CMAKE_FIND_PACKAGE_NAME}_FIND_QUIETLY)
    message(STATUS "Found nvpl_scalapack: (Version:${nvpl_scalapack_VERSION}
                          CMakePackageDir:${CMAKE_CURRENT_LIST_DIR}
                          IncludeDir:${nvpl_scalapack_INCLUDE_DIR}
                          LibraryDir:${nvpl_scalapack_LIBRARY_DIR}
                          IntTypes:${NVPL_SCALAPACK_INT_TYPES}
                          MpiInterfaces:${NVPL_BLACS_MPI_INTERFACES})")
endif()
